/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the mozilla-thunderbird debian package.
 *
 * The Initial Developer of the Original Code is Alexander Sack
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Alexander Sack <asac@debian.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const localDir = Components.classes['@mozilla.org/file/local;1'].
	createInstance().QueryInterface
	(Components.interfaces.nsILocalFile);

const KEY_PROFILEDIR                  = "ProfD";
const KEY_APPDIR                      = "XCurProcD";
var PROFILE_DIR;


// Returns the specified directory hierarchy under the special directory 
// specified by aKey, creating directories along the way if necessary.
function getDir(aKey, aSubDirs)
{
  return getDirInternal(aKey, aSubDirs, true);
}

function getDirNoCreate(aKey, aSubDirs)
{
  return getDirInternal(aKey, aSubDirs, false);
}

function getDirInternal(aKey, aSubDirs, aCreate)
{
  var fileLocator = Components.classes["@mozilla.org/file/directory_service;1"]
                              .getService(Components.interfaces.nsIProperties);
  var dir = fileLocator.get(aKey, Components.interfaces.nsIFile);
  for (var i = 0; i < aSubDirs.length; ++i) {
    dir.append(aSubDirs[i]);
    if (aCreate && !dir.exists())
      dir.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
  }
  return dir;
}

function getFile(aKey, aPathToFile)
{
  var subdirs = [];
  for (var i = 0; i < aPathToFile.length - 1; ++i)
    subdirs.push(aPathToFile[i]);
  var file = getDir(aKey, subdirs);
  file.append(aPathToFile[aPathToFile.length - 1]);
  return file;
}

var debCleanCompreg = {
   onProfileStartup: function() {
      dump("observe: \n"); 

   },

   observe: function(aSubject, aTopic, aData)
   {
    dump("observe: \n"); 
    if (aTopic == "app-startup") {
        dump("nothing here: "+aData+"\n");

	var observerService =
	  Components.classes["@mozilla.org/observer-service;1"]
	    .getService(Components.interfaces.nsIObserverService);

	var observer = {
	  observe: function(subject,topic,data){
		dump("observe called\n");
		try {
		   var file = getFile(KEY_PROFILEDIR,["compreg.dat"]);
		   var extensionsFile = getFile(KEY_APPDIR,["extensions","Extensions.rdf"]);
		   if(!file.exists()) {
		   	return;
		   }
		   if(!extensionsFile.exists()) {
		   	dump("WEIRD: The Global Extensions.rdf does not exists \n => Post a bug \n\n");
		   }
		   if(file.lastModifiedTime < extensionsFile.lastModifiedTime) {
		   	dump("New Global extensions installed.\n");
			file.remove(false);

			Components.classes['@mozilla.org/embedcomp/prompt-service;1']
				.getService(Components.interfaces.nsIPromptService).alert(null, "Please Restart", 
				 "Changes in the set of globally installed extensions detected.\n "+
				 "You must restart Thunderbird one more time in order to finalize those changes!");
		   }
		   
		} catch(e) {
		  dump("exc: "+e+"\n");
		}
		dump("FILE: "+file);
	  }
	};
	

	observerService.addObserver(observer,"profile-after-change",false);

      } else {
	dump("hehe \n");
      }
   },

   QueryInterface: function(aIID)
   {
    if (!aIID.equals(Components.interfaces.nsIObserver) && 
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    return this;
   }
}


var nsIdmModule = 
{
  mClassName:     "DEBCleanCompreg",
  mContractID:    "@mozilla.debian.org/cleanCompreg;1",
  mClassID:       Components.ID("0f59a758-b509-46e5-9bd2-139bc2d5a32d"),

  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aCID.equals(this.mClassID))
      throw Components.results.NS_ERROR_NO_INTERFACE;
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    return this.mFactory;
  },

  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {

    dump(" ***** Registering: Clean Compreg! ****\n");

    aCompMgr = aCompMgr.QueryInterface(
                 Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(this.mClassID, this.mClassName, 
    	this.mContractID, aFileSpec, aLocation, aType);

    // Receive startup notification.
    // We are |getService()|d at app-startup (before profile selection)
    this.getCategoryManager().addCategoryEntry("app-startup", 
      "DEBCleanCompreg", "service," + this.mContractID, true, true);
  },

  unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
  {
    aCompMgr = aCompMgr.QueryInterface(
                 Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(this.mClassID, aFileSpec);

    this.getCategoryManager().deleteCategoryEntry("app-startup", 
      "DEBCleanCompreg", true);
  },

  canUnload: function(aCompMgr)
  {
    return true;
  },

  getCategoryManager: function()
  {
    return Components.classes["@mozilla.org/categorymanager;1"].
      getService(Components.interfaces.nsICategoryManager);
  },

  //////////////////////////////////////////////////////////////////////
  //
  //   mFactory : nsIFactory
  //
  //////////////////////////////////////////////////////////////////////
  mFactory:
  {
    createInstance: function(aOuter, aIID)
    {
      if (aOuter != null)
        throw Components.results.NS_ERROR_NO_AGGREGATION;
      // return the singleton 

      return debCleanCompreg.QueryInterface(aIID);
    },

    lockFactory: function(aLock)
    {
      // quiten warnings
    }
  }
}


function NSGetModule(aCompMgr, aFileSpec)
{
  return nsIdmModule;
}


